/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.buv.inovat.lib.aktionen;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.inovat.lib.funktionen.datei.DateiFunktionen;
import de.inovat.buv.inovat.lib.gui.GuiFunktionen;

import org.eclipse.jface.action.Action;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.ui.IViewPart;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.ViewPart;
import org.jdesktop.jdic.desktop.Desktop;
import org.jdesktop.jdic.desktop.DesktopException;

import java.io.File;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.FILESEPARATOR;

//~ KLASSEN ===================================================================

/**
 * Klasse enthlt die fr die Standardaktionen relevanten Funktionen.
 *
 * Die Standardaktionen werden als Windows-Standardaktionen implementiert.
 * Daher muss man fr bestimmte Datei-Typen (*.txt, *.csv ...) im Windows-Explorer Einstellungen fr 'print'
 * und 'open' definieren.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class StandardAktionen {

    /** Definiert StandardWindowsAktion 'Datei drucken'. */
    public static final int DRUCKEN = 2;

    /** Definiert StandardWindowsAktion 'Datei ffnen'. */
    public static final int OEFFNEN = 1;

    //~ METHODEN ==============================================================

    /**
     * Rufe die StandardWindows-Aktion fr  die Datei.
     * Falls Parameter 'alsTxtKopie' auf true gesetzt wird,
     * wird die Datei zuerst kopiert (in temporren Ordner als Typ 'dateiTyp')
     * und dann wird diese Kopie weiter bearbeitet.
     *
     * @param aktion Die aufzurufende Windows-Aktion. Mgliche Varianten:
     * OEFFNEN - ffne die Datei,
     * DRUCKEN - drucke die Datei.
     *
     * @param dateiName Die zu bearbeitende Datei.
     *
     * @param alsKopie false - wenn die im Parameter 'dateiName' bergebene Datei bearbeitet werden soll,
     * true - wenn die Datei zuerst in temporren Ordner mit dem Endung 'dateiTyp' kopiert werden soll
     * und dann Kopie weiter bearbeitet wird.
     *
     * @param dateiTyp Endung der KopieDatei. Relevant, wenn alsKopie auf true gesetzt ist.
     *
     * @param tempOrdner Ordner, wo die temporre Datei erstellt wird.
     * 
     * @return true, falls die Aktion erfolgreich war, false - sonst.
     */
    public static boolean aufrufenStandardWindowsAktionFuerDatei(int aktion, String dateiName, boolean alsKopie, String dateiTyp, String tempOrdner) {
        boolean isErfolgreich      = true;
        boolean isKopieErfolgreich = true;
        File    datei              = new File(dateiName);

        // Definiere Datei:
        if (datei.exists()) {
            File bearbeitungsDatei = datei;

            // Falls als Kopie, die Datei zuerst als Datei vom bestimmten Typ in temporren Ordner kopieren:
            if (alsKopie) {
                String    tempDatei = tempOrdner + FILESEPARATOR + "KopieVon_" + datei.getName() + dateiTyp;
                Exception e         = DateiFunktionen.kopiereDatei(dateiName, tempDatei);

                if (e == null) {
                    bearbeitungsDatei = new File(tempDatei);
                } else {
                    isKopieErfolgreich = false;
                    Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
                }
            }

            try {
                if (isKopieErfolgreich) {

                    // Datei ffnen:
                    if (aktion == OEFFNEN) {
                        Desktop.open(bearbeitungsDatei);
                    }

                    // Datei drucken:
                    else if (aktion == DRUCKEN) {
                        if (Desktop.isPrintable(bearbeitungsDatei)) {
                            Desktop.print(bearbeitungsDatei);
                        } else {
                            isErfolgreich = false;
                            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Datei <" + dateiName + "> kann nicht ausgedruckt werden."));
                        }
                    }
                } else {
                    isErfolgreich = false;
                    Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Fehler beim Kopieren der Datei <" + dateiName + ">"));
                }
            }
            catch (DesktopException e) {
                isErfolgreich = false;
                Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, "Fehler beim ffnen/Drucken der Datei", e);
            }
        } else {
            isErfolgreich = false;
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Datei <" + dateiName + "> existiert nicht."));
        }

        return isErfolgreich;
    }

    /**
     * Definiere und registriere fr das View die StandardPrintAktion.
     *
     * @param view View, wo die Aktion registriert werden soll.
     * @param viewDatei Ermittelt die DruckDatei.
     * @param alsKopie false - wenn die Datei gedruckt werden soll,
     * true - wenn die Datei in temporren Ordner mit der Endung dateiTyp kopiert werden soll und dann Kopie drucken.
     * @param dateiTyp Endung der KopieDatei. Relevant, wenn alsKopie auf true gesetzt ist.
     * @param istErlaubt Legt fest, ob die Aktion am Anfang erlaubt ist.
     * @param tempOrdner Ordner, wo die temporre Datei erstellt wird.
     */
    public static void registriereActionPrintFuerView(final ViewPart view, final IDateiFuerStandardWindowsAktionen viewDatei, final boolean alsKopie, final String dateiTyp, boolean istErlaubt, final String tempOrdner) {

        // Definiere PrintAktion:
        IAction printAction = new Action() {
            @Override
            public void run() {
                if (!aufrufenStandardWindowsAktionFuerDatei(DRUCKEN, viewDatei.ermittleDateiName(), alsKopie, dateiTyp, tempOrdner)) {
                    MessageDialog.openError(view.getViewSite().getShell(), "Fehler", "Es ist ein Fehler beim Drucken aufgetreten. Siehe Meldung unten.");
                }
            }
        };

        // Registriere fr das View PrintAktion:
        view.getViewSite().getActionBars().setGlobalActionHandler(ActionFactory.PRINT.getId(), printAction);
        printAction.setEnabled(istErlaubt);
        view.getViewSite().getActionBars().updateActionBars();
    }

    /**
     * Setzte im View den Enabled-Wert fr die StandardAktion Print.
     *
     * @param enabled Neuer Enabled-Wert.
     * @param viewID ID der Sicht.
     */
    public static void setzteStandardAktionPrintEnabled(String viewID, boolean enabled) {
        IViewPart view = GuiFunktionen.ermittleView(viewID);

        if (view != null) {
            IAction aktion = view.getViewSite().getActionBars().getGlobalActionHandler(ActionFactory.PRINT.getId());

            if (aktion != null) {
                aktion.setEnabled(enabled);
            }
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 22.03.10
