/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.inovat.lib.debug;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.Activator;

import org.eclipse.core.runtime.*;

import java.util.List;

//~ KLASSEN ===================================================================

/**
 * WICHTIG: Abhngiger Plugin org.eclipse.ui.views.log notwendig.
 * Hier ist die Log-Sicht definiert.
 * 
 * Die Klasse definiert die Meldungen fr das StandartView "ErrorLog".
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class Log {

    /**
     * ID vom Standard-View "ErrorLog".
     */
    public static final String LogViewID = "org.eclipse.pde.runtime.LogView";

    /** Systemunabhngiger Zeilenumbruch. */
    public final static String NEWLINE = System.getProperty("line.separator");

    /**
     * Definiert den Status "OK". Gleich 0 ist.
     */
    public static final int OK = IStatus.OK;

    /**
     * Definiert den Status "INFO". Gleich 1 ist.
     */
    public static final int INFO = IStatus.INFO;

    /**
     * Definiert den Status "WARNUNG". Gleich 2 ist.
     */
    public static final int WARNUNG = IStatus.WARNING;

    /**
     * Definiert den Status "FEHLER_". Gleich 4 ist.
     */
    public static final int FEHLER = IStatus.ERROR;

    /**
     * Definiert den Status "ABBRUCH". Gleich 8 ist.
     */
    public static final int ABBRUCH = IStatus.CANCEL;

    //~ METHODEN ==============================================================

    /**
     * Ermittle den Text fr den Status.
     *
     * @param status Status. Kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log._FEHLER, Log.ABBRUCH sein.
     *
     * @return Text fr den Status.
     */
    public static String ermittleStatusText(int status) {
        switch (status) {
        case ABBRUCH :
            return "ABBRUCH";

        case FEHLER :
            return "FEHLER";

        case INFO :
            return "INFO";

        case OK :
            return "OK";

        case WARNUNG :
            return "WARNUNG";
        }

        return "";
    }

    /**
     * Erzeugt eine Meldung fr "ErrorLog".
     *
     * @param typ kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER_, Log.ABBRUCH sein.
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung.
     *
     * @return die erzeugte Meldung von Typ IStatus.
     */
    public static IStatus erzeugeMeldung(int typ, String pluginID, String meldungText) {
        return new Status(typ, pluginID, meldungText);
    }

    /**
     * Erzeugt eine Meldung fr "ErrorLog".
     *
     * @param typ kann eine der Konstanten Log.OK, Log.INFO, Log.WARNUNG, Log.FEHLER_, Log.ABBRUCH sein.
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung.
     * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
     *
     * @return die erzeugte Meldung von Typ Status.
     */
    public static IStatus erzeugeMeldung(int typ, String pluginID, String meldungText, Throwable exception) {
        return new Status(typ, pluginID, meldungText, exception);
    }

    /**
     * Erzeugt eine MultiMeldung ohne Kinder fr "ErrorLog".
     * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
     * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung.
     *
     * @return die erzeugte MultiMeldung von Typ MultiStatus.
     */
    public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText) {
        return new MultiStatus(pluginID, 0, meldungText, null);
    }

    /**
     * Erzeugt eine MultiMeldung fr "ErrorLog" mit der Kinder.
     * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
     * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     *
     * @param meldungText Text der Meldung.
     * @param kinder Liste der Meldungen. Als Kind kann auch MultiMeldung sein.
     *
     * @return die erzeugte MultiMeldung von Typ MultiStatus.
     */
    public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, List<IStatus> kinder) {
        return new MultiStatus(pluginID, 0, kinder.toArray(new IStatus[kinder.size()]), meldungText, null);
    }

    /**
     * Erzeugt eine MultiMeldung ohne Kinder fr "ErrorLog".
     * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
     * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung.
     * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
     *
     * @return die erzeugte MultiMeldung von Typ MultiStatus.
     */
    public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, Throwable exception) {
        return new MultiStatus(pluginID, 0, meldungText, exception);
    }

    /**
     * Erzeugt eine MultiMeldung fr "ErrorLog" mit der Kinder.
     * Die Kinder knnen mit der Methode add oder addAll hinzugefgt werden.
     * Die MultiMeldung kriegt den schlimmsten Status seiner Kinder.
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung.
     * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
     * @param kinder Liste der Meldungen. Als Kind kann auch MultiMeldung sein.
     *
     * @return die erzeugte MultiMeldung von Typ MultiStatus.
     */
    public static MultiStatus erzeugeMultiMeldung(String pluginID, String meldungText, Throwable exception, List<IStatus> kinder) {
        return new MultiStatus(pluginID, 0, kinder.toArray(new IStatus[kinder.size()]), meldungText, exception);
    }

    /**
     * Stellt fest, ob die Meldung den Status '_Fehler' hat.
     *
     * @param meldung Die zu prfende Meldung.
     *
     * @return true, falls die Meldung den Status '_Fehler' hat,
     * false - sonst.
     */
    public static boolean istFehler(IStatus meldung) {
        return meldung.getSeverity() == Log.FEHLER;
    }

    /**
     * Stellt fest, ob die Meldung den Status 'OK' hat.
     *
     * @param meldung Die zu prfende Meldung.
     *
     * @return true, falls die Meldung den Status 'OK' hat,
     * false - sonst.
     */
    public static boolean istOk(IStatus meldung) {
        return meldung.getSeverity() == Log.OK;
    }

    /**
     * Stellt fest, ob die Meldung den Status 'Warnung' hat.
     *
     * @param meldung Die zu prfende Meldung.
     *
     * @return true, falls die Meldung den Status 'Warnung' hat,
     * false - sonst.
     */
    public static boolean istWarnung(IStatus meldung) {
        return meldung.getSeverity() == Log.WARNUNG;
    }

    /**
     * Zeigt die Meldung im "ErrorLog".
     *
     * @param meldung die zu zeigende Meldung.
     */
    public static void zeige(IStatus meldung) {
    	try {
    		Activator.getDefault().getLog().log(meldung);
		} catch (Exception e) {
		    System.err.println(e);
		}
    }

    /**
     * Zeigt die Meldungen im "ErrorLog".
     *
     * @param listeMeldungen Liste der Meldungen, die angezeigt werden sollen.
     */
    public static void zeige(List<IStatus> listeMeldungen) {
        for (IStatus meldung : listeMeldungen) {
            zeige(meldung);
        }
    }

    /**
     * Erzeugt die Meldung der Typ Warnung, mit dem Text "Interne Systemmeldung"
     * und Exception und zeigt die.
     * Wird fr die interne Meldungen benutzt, die fr den Entwickler relevant sind,
     * um mgliche Fehler_ nachvollziehen zu knnen (z.B. beim Exception im try-catch Block
     * oder man kann selber eine Exception erzeugen: new Throwable(...)).
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param exception Exception, das beim Doppelklick auf der Meldung angezeigt wird.
     */
    public static void zeigeInterneMeldung(String pluginID, Throwable exception) {
        Log.zeige(Log.erzeugeMeldung(Log.WARNUNG, pluginID, "Interne Systemmeldung", exception));
    }

    /**
     * Erzeugt die MultiMeldung mit dem Text "Interne Systemmeldung" und einem Kind der Typ Warnung
     * mit dem im Parameter angegebenen Text und zeigt die.
     * Wird fr die interne Meldungen benutzt, die fr Entwickler relevant sind,
     * um mgliche Fehler_ nachvollziehen zu knnen (z.B. wenn man einen Text beim Fehler_ anzeigen will).
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung von Kind.
     */
    public static void zeigeInterneMultiMeldung(String pluginID, String meldungText) {
        MultiStatus meldung = erzeugeMultiMeldung(pluginID, "Interne Systemmeldung");

        meldung.add(erzeugeMeldung(Log.WARNUNG, pluginID, meldungText));
        Log.zeige(meldung);
    }

    /**
     * Erzeugt die MultiMeldung mit dem Text "Interne Systemmeldung", mit Exception und einem Kind der Typ Warnung
     * mit dem im Parameter angegebenen Text und zeigt die.
     * Wird fr die interne Meldungen benutzt, die fr Entwickler relevant sind,
     * um mgliche Fehler_ nachvollziehen zu knnen (z.B. beim Exception im try-catch Block und
     * wenn man einen Text beim Fehler_ anzeigen will).
     *
     * @param pluginID ID von PlugIn, das die Meldung erzeugt hat. Wird oft ber Activator.PLUGIN_ID erreichbar.
     * @param meldungText Text der Meldung von Kind.
     * @param exception Exception.
     */
    public static void zeigeInterneMultiMeldung(String pluginID, String meldungText, Throwable exception) {
        MultiStatus meldung = erzeugeMultiMeldung(pluginID, "Interne Systemmeldung", exception);

        meldung.add(erzeugeMeldung(Log.WARNUNG, pluginID, meldungText));
        Log.zeige(meldung);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 11.05.10
