/*
 * Copyright (c) 2007 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package de.inovat.buv.inovat.lib.io;

//~ NICHT JDK IMPORTE =========================================================

// import de.bsvrz.sys.funclib.debug.Debug;
import de.inovat.buv.inovat.lib.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

//~ JDK IMPORTE ===============================================================

import java.io.*;

import java.util.*;

//~ KLASSEN ===================================================================

/**
 * Ermglicht den lesenden Zugriff auf Excel-Tabellen im *.csv-Format.
 * <p/>
 * Eine Klasse, mit der mit einem Trennzeichen separierte Werte eingelesen werden knnen. Fr jede Zeile werden die
 * Werte in einem Hashtable abgelegt. Eine Klasse, die die Daten weiterverarbeiten soll, kann dann unter dem Namen der
 * jeweiligen Spalte die Daten aus dem Hashtable auslesen.
 *
 * @author inovat
 * @author Dipl.-Ing. Hans Chrstian Kni (HCK)
 * @version $Revision: 12 $ / $Date: 2013-09-24 13:23:48 +0200 (Di, 24. Sep 2013) $ / ($Author: LG $)
 */
public class CsvLeser {

    /** Inhalt der aktuell gelesenen Zeile */
    private String _naechsteZeile = null;

    // ------------------------------------------------------------------------
    // VARIABLENDEKLARATIONEN
    // ------------------------------------------------------------------------
    // Private ----------------------------------------------------------------

    /** DebugLogger fr Debug-Ausgaben. */
    // private static final Debug debug = Debug.getLogger();

    /** Trennzeichen (i.d.R. ';'), durch welches einzelne Zellen innerhalb einer Zeile getrennt sind. */
    private char _begrenzungsZeichen;

    /** Reader, mit der die CSV-Datei gelesen wird. */
    private BufferedReader _csvLeser;

    /** Inhalt der ersten Zeile. */
    private List<String> _ersteZeile;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    // ------------------------------------------------------------------------
    // KONSTRUKTOREN  (und vom Konstruktor verwende Methoden)
    // ------------------------------------------------------------------------

    /**
     * Konstruktor fr den CSV-Datei-Leser.
     *
     * @param leser              Der zu verwendende Leser.
     * @param begrenzungsZeichen Trennzeichen (i.d.R. ';'), durch welches einzelne Zellen innerhalb einer Zeile getrennt
     *                           sind.
     */
    public CsvLeser(Reader leser, char begrenzungsZeichen) {
        this._begrenzungsZeichen = begrenzungsZeichen;
        this._csvLeser           = new BufferedReader(leser);
        this._ersteZeile         = ermitteleSpaltenNamen();
    }

    //~ METHODEN ==============================================================

    /** Schliet die *.csv-Datei. */
    public void close() {
        try {
            _csvLeser.close();
        }

        // kann ignoriert werden, da hasNochMehrZeilen den Reader bereits schliesst
        catch (IOException e) {

            // debug.finest("Datei bereits geschlossen (eventuell schon letzte Zeile eingelesen: " + e);
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }
    }

    /**
     * Ermittelt die Spaltennamen der Tabelle (Zelleneintrge der ersten Zeile).
     *
     * @return Spaltennamen der Tabelle.
     */
    private List<String> ermitteleSpaltenNamen() {
        List<String> spaltenNamen = null;

        try {
            String zeile = _csvLeser.readLine();

            spaltenNamen = ermittleZellenEinerZeile(zeile);
        }
        catch (Exception e) {
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        return spaltenNamen;
    }

    /**
     * Zerlegt die Zeile in die einzelnen Zellen unter Bercksichtigung von Anfhrungszeichen in Zellen
     *
     * @param zeile aktuelle Zeile der *.csv-Datei.
     *
     * @return Liste mit den Zelleninhalten.
     */
    private List<String> ermittleZellenEinerZeile(String zeile) {
        List<String> zellen = new Vector<String>();
        boolean      anfuehrungsZeichen;
        int          start, ende,
                     index           = 0,
                     maxZeilenLaenge = zeile.length() - 1;

        try {

            // Alle Spalten durchlaufen
            while (index <= maxZeilenLaenge) {
                start              = index;
                anfuehrungsZeichen = false;

                // Inhalt einer Spalte extrahieren
                while (index <= maxZeilenLaenge) {
                    char pruefZeichen = zeile.charAt(index);

                    // Nun wird der nchste Delimiter gesucht, der NICHT
                    // innerhalb von Anfhrungszeichen (") steht. Wenn ein
                    // Anfhrungszeichen gefunden wurde, dann muss der Merker
                    // getoggled werden.
                    if (pruefZeichen == '"') {
                        anfuehrungsZeichen = !anfuehrungsZeichen;

                        // Es befindet sich auf jeden Fall eine gerade Anzahl von
                        // "-Zeichen zwischen den Quotes, so dass nur die am Anfang
                        // und Ende bercksichtigt werden.
                    } else if ((pruefZeichen == _begrenzungsZeichen) &&!anfuehrungsZeichen) {

                        // noinspection BreakStatement
                        break;
                    }

                    index++;
                }

                ende = index;

                // Anfhrungszeichen am Anfang und am Ende gehren nicht zum
                // String und werden deshalb auch nicht beachtet.
                if ((zeile.charAt(start) == '"') && (zeile.charAt(ende - 1) == '"')) {
                    start++;
                    ende--;
                }

                // Der gefundene Text wird in dem Vector gespeichert, ohne jedoch zu
                // vergessen, dass zwei auf einander folgende Anfhrungszeichen ("")
                // durch ein Einzelnes (") zu ersetzen sind.
                zellen.add(zeile.substring(start, ende).replaceAll("\"\"", "\""));
                index++;
            }
        }
        catch (Exception e) {

            // debug.error("_Fehler beim zerlegen einer Zeile der *.csv-Datei: " + e, e);
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        return zellen;
    }

    //~ GET METHODEN ==========================================================

    // ------------------------------------------------------------------------
    // GETTER / SETTER METHODEN
    // ------------------------------------------------------------------------

    /**
     * Liefert die erste Zeile der *.csv-Datei mit der Liste der Spaltennamen.
     *
     * @return Liste der Spaltennamen aus der ersten Zeile der *.csv-Datei.
     */
    public List<String> getErsteZeile() {
        return _ersteZeile;
    }

    // ------------------------------------------------------------------------
    // SONSTIGE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Liefert die Zellen einer Zeile.
     *
     * @return Die Zellen einer Zeile in einer Map. Schlssel ist der Spaltenname, Objekt ist der Inhalt der entsprechenden
     *         Zelle der aktuellen Zeile.
     *         <p/>
     *         Ist keine Zeile mehr vorhanden, wird eine leere Map zurckgeliefert.
     */
    public Map<String, String> getNaechsteZeile() {

        // Map mit den Zellen einer Zeile
        Map<String, String> zellenDerZeile = new HashMap<String, String>();

        // Liest auf jeden Fall die neue Zeile, wenn es eine gibt.

        if (hasNochMehrZeilen()) {

            // Aus der Zeile wird die Hashtable erzeugt.
            List<String> zellen = ermittleZellenEinerZeile(_naechsteZeile.trim());

            for (int i = zellen.size() - 1; i >= 0; i--) {
                zellenDerZeile.put(_ersteZeile.get(i), zellen.get(i));
            }

            // Lscht die Zeile, damit hasNochMehrZeilen auf jeden Fall
            // eine neue Zeile einliest.
            _naechsteZeile = "";
        }

        return zellenDerZeile;
    }

    /**
     * Prft, ob die *.csv-Datei noch weitere Zeilen zum EInlesen hat.
     *
     * @return <code>true</code>, wenn noch Zeilen eingelesen werden knnen, sonst <code>false</code>.
     */
    public boolean hasNochMehrZeilen() {
        boolean hasNochMehrZeilen;

        try {
            if ((_naechsteZeile == null) || _naechsteZeile.trim().equals("")) {
                _naechsteZeile = _csvLeser.readLine();
            }
        }
        catch (Exception e) {

            // debug.finest("Ignorierte Exception beim Einlesen: " + e);
            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
        }

        if ((_naechsteZeile == null) || _naechsteZeile.trim().equals("")) {
            close();
            hasNochMehrZeilen = false;
        } else {
            hasNochMehrZeilen = true;
        }

        return hasNochMehrZeilen;
    }

    //~ METHODEN ==============================================================

    // ------------------------------------------------------------------------
    // KANONISCHE METHODEN
    // ------------------------------------------------------------------------

    /**
     * Gibt die String-Reprsentation dieses Objekts aus.
     * <p/>
     * Die Formatierung der Ausgabe kann sich ndern und wird nicht garantiert.
     *
     * @return Aktueller Zustand der Klassenvariablen dieses Objekts.
     */
    public String toString() {
        return "CsvLeser{" + "_begrenzungsZeichen=" + _begrenzungsZeichen + ", _csvLeser=" + _csvLeser + ", _ersteZeile=" + _ersteZeile + ", _naechsteZeile='" + _naechsteZeile + '\'' + '}';
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 29.05.09
