/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.bast.gui;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.projektlib.speicher.GrundEinstellungen;
import de.inovat.buv.plugin.gtm.bast.Activator;
import de.inovat.buv.inovat.lib.debug.Log;

import java.beans.XMLDecoder;
import java.beans.XMLEncoder;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.util.*;

//~ KLASSEN ===================================================================

/**
 * Verwaltet Parameter fr den Export und ermglicht den zentralen Zugriff
 * auf Parameter sowie auf parameterbergreifende allgemeine Eigenschaften.
 *
 * Fr jeden Export wird einzige Instanz der Klasse erzeugt und im Map gespeichert.
 * Als Schlssel im Map dient ID der Sicht.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 359 $ / $Date: 2013-03-12 11:07:20 +0100 (Di, 12 Mrz 2013) $ / ($Author: LG $)
 */
public final class ParameterVerwaltung {

    /**
     * Map von Instanzen der Klasse. Als Schlssel dient ID der Sicht.
     * Der Zugriff auf die Klasse und ihre Methoden erfolgt ber diese Instanzen.
     */
    private static final Map<String, ParameterVerwaltung> mapParVewInstanz = new HashMap<String, ParameterVerwaltung>();

    /**
     * Map von Mengen der fr Parameternderungen registrierten Klassen.
     * Als Schlssel dient ID der Sicht.
     */
    private static final Map<String, Set<IParameterAenderung>> mapMengeParameterAenderungsListener = new HashMap<String, Set<IParameterAenderung>>();

    /**
     * Default ExportDatei. Wird benutzt, falls der Name der ExportDatei im Parameter leer ist.
     */
    private static String _exportDatei;

    /**
     * ID des Parameters, ber das die String-Representation des Parameters aus dem Speicher abgeholt wird.
     */
    private static String _parameterID;

    /**
     * ID der Sicht.
     * Dient als Schlssel im Map von Instanzen der ParameterVerwaltung
     * und im Map von Mengen der als Listener auf die ParameterAenderungs registrierten Klassen.
     */
    private static String _viewID;

    //~ FELDER ================================================================

    /**
     * Instanz der Klasse Parameter.
     */
    private Parameter _parameter;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse ParameterVerwaltung.
     * Fr jede ParameterDatei wird eine einzige Instanz erzeugt.
     *
     */
    private ParameterVerwaltung() {
        _parameter = initParameter();
    }

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IParameterAenderung}.
     *
     * @param viewID ID der Sicht.
     * @param klasse Klasse, die das Interface {@link IParameterAenderung} implementiert
     * und ber Parameternderungen informiert werden will.
     */
    protected static void addParameterListener(String viewID, IParameterAenderung klasse) {
    	synchronized (mapMengeParameterAenderungsListener) {
    		Set<IParameterAenderung> mengeParameterAenderungsListener = mapMengeParameterAenderungsListener.get(viewID);
    		
    		if (mengeParameterAenderungsListener == null) {
    			mengeParameterAenderungsListener = new HashSet<IParameterAenderung>();
    			mapMengeParameterAenderungsListener.put(viewID, mengeParameterAenderungsListener);
    		}
    		
    		mengeParameterAenderungsListener.add(klasse);
		}
    }

    /**
     * Ruft bei nderungen in den Parametern die Methode {#link IParameterAenderung.parameterGeaendert}() fr jede als
     * Listener registrierte Klasse auf.
     */
    private static void benachrichtigeParameterAenderungsListener() {
    	synchronized (mapMengeParameterAenderungsListener) {
    		Set<IParameterAenderung> mengeParameterAenderungsListener = mapMengeParameterAenderungsListener.get(_viewID);
    		
    		if (mengeParameterAenderungsListener != null) {
    			for (IParameterAenderung klasse : mengeParameterAenderungsListener) {
    				klasse.parameterGeaendert();
    			}
    		}
    	}
    }

    //~ GET METHODEN ==========================================================

    /**
     * Gibt die Instanz der Klasse ParameterVerwaltung fr die angegebene Datei zurck.
     * <p/>
     * Die ParameterVerwaltung ist als Singleton realisiert
     * und kann nicht direkt instanziert werden. Der Zugriff auf die
     * Methoden erfolgt ausschlielich ber die Instanz.
     *
     * @param viewID ID der Sicht. Wird im Map von Instanzen der Klasse ParameterVerwaltung und
     * im Map von Mengen der fr Parameternderungen registrierten Klassen als Schlssel dienen.
     * @param parameterID ID des Parameters, ber das die String-Representation des Parameters aus dem Speicher abgeholt wird.
     * @param defaultExportDatei Default ExportDatei.
     *
     * @return Instanz der Klasse ParameterVerwaltung fr die angegebene Datei.
     */
    protected static ParameterVerwaltung getInstanz(String viewID, String parameterID, String defaultExportDatei) {
        _viewID      = viewID;
        _parameterID = parameterID;
        _exportDatei = defaultExportDatei;

        ParameterVerwaltung parVewInstanz = mapParVewInstanz.get(viewID);

        if (parVewInstanz == null) {
            parVewInstanz = new ParameterVerwaltung();
            mapParVewInstanz.put(viewID, parVewInstanz);
        }

        return parVewInstanz;
    }

    /**
     * Getter fr Eigenschaft 'parameter'.
     *
     * @return Wert der Eigenschaft 'parameter'.
     *
     * @see #_parameter
     */
    protected Parameter getParameter() {
        return _parameter;
    }

    //~ METHODEN ==============================================================

    /**
     * Holt den Parameter von dem Speicher ab.
     *
     * @return Im Speicher gespeicherter Parameter.
     */
    private Parameter initParameter() {
        Parameter parameter = new Parameter("", 0, "");

        // Ermittle die String-Representation des Parameters aus dem Speicher:
        String parString = GrundEinstellungen.ermittleParameterWertFuerBenutzer(_parameterID);

        if (parString != null) {
            try {

                // Dekodiere den Parameter:
                ByteArrayInputStream input     = new ByteArrayInputStream(parString.getBytes());
                XMLDecoder           xml       = new XMLDecoder(input);
                Object               parObjekt = xml.readObject();

                xml.close();

                if (parObjekt instanceof Parameter) {
                    parameter = (Parameter) parObjekt;
                }
            }
            catch (Exception e) {
                parameter = new Parameter("", 0, "");
                Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e);
            }
        }

        if (parameter.getDatei().equals("")) {
            parameter.setDatei(_exportDatei);
        }

        parameter.setFortschrittsanzeige(0);
        parameter.setMeldungen("");

        return parameter;
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf {@link IParameterAenderung}.
     *
     * @param viewID ID der Sicht.
     * @param klasse Klasse, die sich zuvor mittels {#link addParameterListener} registriert hat,
     */
    protected static void removeParameterListener(String viewID, IParameterAenderung klasse) {
    	synchronized (mapMengeParameterAenderungsListener) {
    		Set<IParameterAenderung> mengeParameterAenderungsListener = mapMengeParameterAenderungsListener.get(viewID);
    		
    		if (mengeParameterAenderungsListener != null) {
    			mengeParameterAenderungsListener.remove(klasse);
    		}
    	}
    }

    /**
     * Speichere den Parameter ber den XMLEncoder.
     *
     * @param par Der neue Parameter.
     */
    protected void speichere(Parameter par) {
        _parameter = par;

        // Kodiere den Parameter ber XMLEncoder:
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        XMLEncoder            xml = new XMLEncoder(out);

        xml.writeObject(_parameter);
        xml.close();
        GrundEinstellungen.speichereParameterFuerBenutzer(_parameterID, out.toString(), "");
    }

    /**
     * Aktualisiere den Parameter und rufe benachrichtigeParameterAenderungsListener() auf.
     * Die Methode update() wird aufgerufen, wenn der Parameter gendert wird.
     *
     * @param par Der genderte Parameter.
     */
    protected void update(Parameter par) {
        if (par != null) {
            _parameter = par;
            benachrichtigeParameterAenderungsListener();
        } else {
            Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, "Parameter der Methode ParameterVerwaltung.update()darf nicht null sein!");
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 23.04.10
