/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.speicher.einstellungen;

//~ NICHT JDK IMPORTE =========================================================

import de.bsvrz.buv.rw.basislib.einstellungen.EinstellungsAdresse;
import de.bsvrz.buv.rw.basislib.einstellungen.SpeicherKey;
import de.bsvrz.buv.rw.basislib.urlasser.DefaultDatenSender;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserDatenDialog;
import de.bsvrz.buv.rw.basislib.urlasser.UrlasserInfo;
import de.bsvrz.dav.daf.main.config.SystemObject;

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.rwservice.RWService;

import org.eclipse.core.runtime.IStatus;
import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

//~ JDK IMPORTE ===============================================================

import java.util.ArrayList;
import java.util.List;

//~ KLASSEN ===================================================================

/**
 * Klasse verwaltet die Speicherfunktionen.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 127 $ / $Date: 2011-07-19 10:22:46 +0200 (Di, 19. Jul 2011) $ / ($Author: LG $)
 */
public class SpeicherFunktionen {

    /**
     * Falls der Speicher synchronisiert werden soll,
     * wird fr jeden Parameter auch Zeitpunkt mitgespeichert
     * (mit der ID = ParameterID + ID_ZEITPUNKT).
     */
    private final static String ID_ZEITPUNKT = ".syncZeitPunkt";

    /**
     * Synchronisation vom SpeicherProzess.
     * Whrend des SpeicherProzesses werden weitere Prozesse
     * (Speichern und Laden mit Synchronisation) gesperrt.
     */
    private static final Boolean speicherProzessSync = true;

    //~ KONSTANTE AUFZAEHLUNGSTYPEN ===========================================

    /**
     * Aufzhlung fr die Art (allgemein/Benutzer) und
     * Ort (DaV/lokal) der Einstellungen.
     */
    public static enum ArtOrt { ALLGEMEIN_LOKAL, ALLGEMEIN_DAV, BENUTZER_LOKAL, BENUTZER_DAV }

    //~ METHODEN ==============================================================

    /**
     * Ermittle den aktuellsten ArtOrt.
     * Falls alle Speicher gleich aktuell sind,
     * wird null zurckgegeben.
     *
     * @param parameterID ID des Parameters
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Aktuellster ArtOrt oder null, falls alle Speicher gleich aktuell sind.
     */
    private static ArtOrt ermittleAktuellsteSpeicherArt(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        int anz = speicherArtOrt.getArArtOrt().length;

        // Lese die Aktualisierungszeitpunkte:
        long[] arZeitpunkt = new long[anz];

        for (int i = 0; i < anz; i++) {
            arZeitpunkt[i] = ermittleDatum(parameterID, speicherArtOrt.getArArtOrt()[i]);
        }

        // Berechne max. Aktualisierungszeitpunkt und zugehrigen Speicherart:
        boolean istDatumGleich = true;
        ArtOrt  aktArtOrt      = speicherArtOrt.getArArtOrt()[0];
        long    maxDatum       = arZeitpunkt[0];

        for (int i = 1; i < anz; i++) {
            if (maxDatum < arZeitpunkt[i]) {
                maxDatum       = arZeitpunkt[i];
                aktArtOrt      = speicherArtOrt.getArArtOrt()[i];
                istDatumGleich = false;
            } else if (maxDatum > arZeitpunkt[i]) {
                istDatumGleich = false;
            }
        }

        // Falls den Aktualisierungszeitpunkt berall gleich ist,
        // sind alle Speicher im aktuellen Zustand:
        if (istDatumGleich) {
            aktArtOrt = null;
        }

        return aktArtOrt;
    }

    /**
     * Ermittle die Beschreibung fr die Speicherart.
     *
     * @param artOrt Art und Ort von Speicher.
     *
     * @return Beschreibung fr die Speicherart.
     */
    private static String ermittleBeschreibung(ArtOrt artOrt) {
        switch (artOrt) {
        case BENUTZER_DAV :
            return "fr den Benutzer global";

        case BENUTZER_LOKAL :
            return "fr den Benutzer lokal";

        case ALLGEMEIN_DAV :
            return "allgemein global";

        case ALLGEMEIN_LOKAL :
            return "allgemein lokal";

        default :
            return "";
        }
    }

    /**
     * Ermittle den Aktualisierungszeitpunkt fr den Parameter und die Speicherart.
     *
     * @param parameterID ID des Parameters.
     * @param artOrt Definiert Art, Ort von Speicher.
     *
     * @return Aktualisierungszeitpunkt fr den Parameter und die Speicherart
     * oder 0.
     */
    private static long ermittleDatum(String parameterID, ArtOrt artOrt) {
        try {
            SpeicherKey speicher = ermittleSpeicher(artOrt);

            // Lese den Aktualisierungszeitpunkt aus:
            if (speicher != null) {
                String datum = RWService.getService().getEinstellungen().getValue(  //
                    new EinstellungsAdresse(null, parameterID + ID_ZEITPUNKT, speicher)).toString();

                return Long.valueOf(datum);
            }
        }
        catch (Exception ignored) {}

        return 0;
    }

    /**
     * Ermittle SpeicherKey der Einstellung.
     *
     * @param artOrt Art und Ort der Einstellung.
     *
     * @return SpeicherKey der Einstellung oder null.
     */
    public static SpeicherKey ermittleSpeicher(ArtOrt artOrt) {
        switch (artOrt) {
        case BENUTZER_DAV :
            return SpeicherKey.benutzerNetzweit();

        case BENUTZER_LOKAL :
            return SpeicherKey.benutzerLokal();

        case ALLGEMEIN_DAV :
            return SpeicherKey.allgemeinNetzweit();

        case ALLGEMEIN_LOKAL :
            return SpeicherKey.allgemeinLokal();

        default :
            return null;
        }
    }

    /**
     * Erstelle die UrlasserInfo mit den Daten des angemeldeten Benutzer und angegebener Ursache.
     *
     * @param ursache Ursache.
     *
     * @return UrlasserInfo.
     */
    public static UrlasserInfo erstelleUrlasserInfo(String ursache) {
        SystemObject benutzer = RWService.getService().getRw().getBenutzer();
        String       passwort = RWService.getService().getRw().getPasswort();

        return new UrlasserInfo(benutzer, passwort, ursache, benutzer.getName());
    }

    /**
     * Ermittle den gespeicherten Parameter.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert oder null.
     */
    public static String ladeParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        String parameter = null;

        if (parameterID != null) {
            SpeicherKey speicher;

            // ------------------------------------------------------------
            // Lade den Parameter mit der Synchronisation:
            if (speicherArtOrt.isSynchronisieren()) {

                // Lade den Parameter mit der Synchronisation:
                ArtOrt  aktArtOrt         = ermittleAktuellsteSpeicherArt(parameterID, speicherArtOrt);
                boolean isSynchronisieren = true;

                // Falls alle Speicher gleich aktuell sind
                // (aktArtOrt==null), braucht man keine Synchronisation:
                if (aktArtOrt == null) {
                    aktArtOrt         = speicherArtOrt.getArArtOrt()[0];
                    isSynchronisieren = false;
                }

                speicher = ermittleSpeicher(aktArtOrt);

                if (speicher != null) {
                    try {
                        parameter = RWService.getService().getEinstellungen().getValue(  //
                            new EinstellungsAdresse(null, parameterID, speicher)).toString();
                    }
                    catch (Exception ignored) {}

                    // Synchronisiere (Speichere Parameter fr alle Speicher aus dem Array):
                    if (isSynchronisieren) {

                        // UrlasserInfo:
                        UrlasserInfo urlasserInfo = erstelleUrlasserInfo(String.format(  //
                            "Synchronisieren des Einstellungsparameters <%s>", parameterID));

                        // speichere:
                        speichere(parameterID, speicherArtOrt, parameter, urlasserInfo);
                    }
                }
            }

            // ------------------------------------------------------------
            // Lade den Parameter ohne die Synchronisation
            // (suche in aller Speicher, falls gefunden - unterbreche):
            else {
                for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                    speicher = ermittleSpeicher(artOrt);

                    if (speicher != null) {
                        try {
                            parameter = RWService.getService().getEinstellungen().getValue(  //
                                new EinstellungsAdresse(null, parameterID, speicher)).toString();
                        }
                        catch (Exception ignored) {}
                    }

                    if (parameter != null) {
                        break;
                    }
                }
            }
        }

        return parameter;
    }

    /**
     * Ermittle den gespeicherten Parameter als Boolean.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Boolean oder null.
     */
    public static Boolean ladeParameterAlsBoolean(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        Boolean boolWert = null;
        String  text     = ladeParameter(parameterID, speicherArtOrt);

        if (text != null) {
            try {
                boolWert = Boolean.valueOf(text);
            }
            catch (Exception e) {
                boolWert = null;
            }
        }

        return boolWert;
    }

    /**
     * Ermittle den gespeicherten Parameter als Integer.
     *
     * @param parameterID ID des Parameters, der ermittelt werden soll.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Definiert Art, Ort von Speicher.
     *
     * @return Parameterwert als Integer oder null.
     */
    public static Integer ladeParameterAlsInteger(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        Integer intWert = null;
        String  text    = ladeParameter(parameterID, speicherArtOrt);

        if (text != null) {
            try {
                intWert = Integer.valueOf(text);
            }
            catch (NumberFormatException e) {
                intWert = 0;
            }
        }

        return intWert;
    }

    /**
     * Lsche den Parameter fr alle Speicherarten.
     * Falls die Variable _synchronisieren aus speicherArtOrt auf true gesetzt ist,
     * wird gleichzeitig den ZeitPunkt gelscht.
     * Am Anfang wird der UrlasserDialog aufgemacht.
     *
     * @param parameterID ID des zu lschenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @return true, falls der Parameter erfolgreich gelscht wurde,
     * false - sonst.
     */
    public static boolean loescheParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt) {
        List<IStatus> listeWarnungen = new ArrayList<IStatus>();

        synchronized (speicherProzessSync) {
            if (parameterID != null) {
                SpeicherKey speicher;

                for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                    speicher = ermittleSpeicher(artOrt);

                    // Lsche:
                    if (speicher != null) {

                        // Lsche den Parameter:
                        try {
                            RWService.getService().getEinstellungen().removeValue(                                                 //
                                new EinstellungsAdresse(null, parameterID, speicher));
                        }
                        catch (Exception e) {
                            listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(                 //
                                "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt)), e));
                        }

                        // Lsche den Parameter mit dem Zeitpunkt (fr den synchronisierten Speicher):
                        try {
                            if (speicherArtOrt.isSynchronisieren()) {
                                RWService.getService().getEinstellungen().removeValue(                                             //
                                    new EinstellungsAdresse(null, parameterID + ID_ZEITPUNKT, speicher));
                            }
                        }
                        catch (Exception e) {
                            listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID + ID_ZEITPUNKT, String.format(  //
                                "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt)), e));
                        }
                    } else {
                        listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(                     //
                            "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt))));
                    }
                }
            }
        }

        if (listeWarnungen.isEmpty()) {
            return true;
        } else {
            Log.zeige(Log.erzeugeMultiMeldung(Activator.PLUGIN_ID, "Fehler beim Speichern des Parameters", listeWarnungen));

            return false;
        }
    }

    /**
     * Lsche den Parameter fr alle Speicherarten.
     * Falls die Variable _synchronisieren aus speicherArtOrt auf true gesetzt ist,
     * wird gleichzeitig den ZeitPunkt gelscht.
     *
     * Achtung: es wird die RW-Methode setValue(einstellung, null, urlasserInfo) aufgerufen,
     * da es keine removeValue(einstellung, urlasserInfo) gibt.
     *
     * @param parameterID ID des zu lschenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gelscht wurde,
     * false - sonst.
     */
    public static boolean loescheParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt, UrlasserInfo urlasserInfo) {
        List<IStatus> listeWarnungen = new ArrayList<IStatus>();

        synchronized (speicherProzessSync) {
            if (parameterID != null) {
                SpeicherKey speicher;

                for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                    speicher = ermittleSpeicher(artOrt);

                    // Lsche:
                    if (speicher != null) {

                        // Lsche den Parameter:
                        try {
                            RWService.getService().getEinstellungen().setValue(                                                        //
                                new EinstellungsAdresse(null, parameterID, speicher), null, urlasserInfo);
                        }
                        catch (Exception e) {
                            listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(                     //
                                "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt)), e));
                        }

                        // Lsche den Parameter mit dem Zeitpunkt (fr den synchronisierten Speicher):
                        if (speicherArtOrt.isSynchronisieren()) {
                            try {
                                RWService.getService().getEinstellungen().setValue(                                                    //
                                    new EinstellungsAdresse(null, parameterID + ID_ZEITPUNKT, speicher), null, urlasserInfo);
                            }
                            catch (Exception e) {
                                listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID + ID_ZEITPUNKT, String.format(  //
                                    "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt)), e));
                            }
                        }
                    } else {
                        listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(                         //
                            "Der Parameter <%s> kann nicht %s gelscht werden!", parameterID, ermittleBeschreibung(artOrt))));
                    }
                }
            }
        }

        if (listeWarnungen.isEmpty()) {
            return true;
        } else {
            Log.zeige(Log.erzeugeMultiMeldung(Activator.PLUGIN_ID, "Fehler beim Speichern des Parameters", listeWarnungen));

            return false;
        }
    }

    /**
     * ffne den Urlasser-Dialog mit vorbelegtem Benutzer-Namen und angegebener Ursache.
     *
     * @param ursache Ursache.
     *
     * @return UrlasserInfo oder null.
     */
    public static UrlasserInfo oeffneUrlasserDialog(String ursache) {
        try {

            // UrlasserDialog:
            final Shell               shell    = PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell();
            final UrlasserDatenDialog dialog   = new UrlasserDatenDialog(shell, new DefaultDatenSender(null));
            String                    benutzer = RWService.getService().getRw().getBenutzerName();

            dialog.setInitialBenutzer(benutzer);
            dialog.setInitialVeranlasser(benutzer);
            dialog.setInitialUrsache(ursache);

            if (dialog.open() == Window.OK) {
                return dialog.getUrlasserInfo();
            }
        }
        catch (Exception ignored) {}

        return null;
    }

    /**
     * berschreibe den Parameter fr alle Speicherarten mit dem neuen Wert.
     * Falls die Variable _synchronisieren aus speicherArtOrt auf true gesetzt ist,
     * wird gleichzeitig den ZeitPunkt gespeichert.
     *
     * _WICHTIG: Whrend dieses Prozesses werden weitere Prozesse
     * (Speichern und Laden mit Synchronisation) gesperrt.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return Warnung als IStatus, falls beim Speichern _Fehler auftreten,
     * null - sonst.
     */
    private static IStatus speichere(String parameterID, SpeicherArtUndOrt speicherArtOrt, String wert, UrlasserInfo urlasserInfo) {
        List<IStatus> listeWarnungen = new ArrayList<IStatus>();

        synchronized (speicherProzessSync) {
            if ((parameterID != null) && (wert != null)) {
                SpeicherKey speicher;
                String      datum = "" + System.currentTimeMillis();

                for (ArtOrt artOrt : speicherArtOrt.getArArtOrt()) {
                    speicher = ermittleSpeicher(artOrt);

                    // Speichere:
                    if (speicher != null) {

                        // Speichere den Parameter:
                        try {
                            RWService.getService().getEinstellungen().setValue(                                         //
                                new EinstellungsAdresse(null, parameterID, speicher), wert, urlasserInfo);
                        }
                        catch (Exception e) {
                            listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(      //
                                "Der Parameter <%s> kann nicht %s gespeichert werden!", parameterID, ermittleBeschreibung(artOrt), e)));
                        }

                        // Speichere den Parameter mit dem Zeitpunkt (fr den synchronisierten Speicher):
                        if (speicherArtOrt.isSynchronisieren()) {
                            try {
                                RWService.getService().getEinstellungen().setValue(                                     //
                                    new EinstellungsAdresse(null, parameterID + ID_ZEITPUNKT, speicher), datum, urlasserInfo);
                            }
                            catch (Exception e) {
                                listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                                    "Der Parameter <%s> kann nicht %s gespeichert werden!", parameterID + ID_ZEITPUNKT, ermittleBeschreibung(artOrt)), e));
                            }
                        }
                    } else {
                        listeWarnungen.add(Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(          //
                            "Der Parameter <%s> kann nicht %s gespeichert werden!", parameterID, ermittleBeschreibung(artOrt))));
                    }
                }
            }
        }

        if (listeWarnungen.isEmpty()) {
            return null;
        } else {
            return Log.erzeugeMultiMeldung(Activator.PLUGIN_ID, "Fehler beim Speichern des Parameters", listeWarnungen);
        }
    }

    /**
     * berschreibe den Parameter fr alle Speicherarten mit dem neuen Wert
     * Falls die Variable _synchronisieren aus speicherArtOrt auf true gesetzt ist,
     * wird gleichzeitig den ZeitPunkt gespeichert.
     * Am Anfang wird der UrlasserDialog aufgemacht.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird,
     * false - sonst.
     */
    public static boolean speichereParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt, String wert) {

        // UrlasserDialog:
        final UrlasserInfo urlasserInfo = oeffneUrlasserDialog(String.format("nderung des Einstellungsparameters <%s>", parameterID));

        // Speichere:
        return speichereParameter(parameterID, speicherArtOrt, wert, urlasserInfo);
    }

    /**
     * berschreibe den Parameter fr alle Speicherarten mit dem neuen Wert
     * Falls die Variable _synchronisieren aus speicherArtOrt auf true gesetzt ist,
     * wird gleichzeitig den ZeitPunkt gespeichert.
     *
     * @param parameterID ID des berschreibenden Parameters.
     * Muss eindeutig sein.
     *
     * @param speicherArtOrt Art und Ort von Speicher.
     *
     * @param wert Neuer Wert fr den Parameter.
     *
     * @param urlasserInfo UrlasserInfo.
     *
     * @return true, falls der Parameter erfolgreich gespeichert wird,
     * false - sonst.
     */
    public static boolean speichereParameter(String parameterID, SpeicherArtUndOrt speicherArtOrt, String wert, UrlasserInfo urlasserInfo) {

        // Pause, damit Puffer schafft alle Datenstze zu schicken
        // (Problem auftritt, wenn mehrere gre Datenstze schnell gesendet werden sollen)
        // System.out.println(String.format("Speichere Parameter: <%s> %s", parameterID, wert));
        try {
            Thread.sleep(1000);
        }
        catch (InterruptedException e) {
            e.printStackTrace();
        }

        // UrlasserDialog:
        IStatus meldung;

        // Speichere:
        if (urlasserInfo != null) {

            // Definiert, ob der Parameter mind. ein Mal gespeichert wird
            meldung = speichere(parameterID, speicherArtOrt, wert, urlasserInfo);
        } else {
            meldung = Log.erzeugeMeldung(Log.WARNUNG, Activator.PLUGIN_ID, String.format(  //
                "Parameter <%s> wird nicht gespeichert (es fehlen die Urlasser-Daten)", parameterID));
        }

        // FehlerMeldung:
        if (meldung != null) {
            Log.zeige(meldung);
        }

        return (meldung == null);
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 26.11.13
