/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.plugin.gtm.navigation.export;

//~ JDK IMPORTE ===============================================================

import java.io.File;

import javax.swing.filechooser.FileFilter;

//~ KLASSEN ===================================================================

/**
 * Erstellt einen neues <code>SimpleFileFilter</code> mit <code>_extension</code>
 * als Extension. _extension kann entweder nur eine Extension sein, z.B. "xml" oder
 * mehrere mit Kommas getrennt: "xml,xsl". Als Description wird automatisch
 * <code>_extension+"-Files (*.ext1, *.ext2)"</code> gesetzt, wobei ext1 und ext2
 * fr Element 1 und Element 2 der Extensionliste stehen. Es knnen natrlich
 * so viele Extensions wie gewnscht gesetzt werden.
 *
 * @author inovat
 * @author Dagmar Rostek (DR)
 * @version $Revision: 155 $ / $Date: 2014-10-07 11:11:48 +0200 (Di, 07 Okt 2014) $ / ($Author: LG $)
 */
public class SimpleFileFilter extends FileFilter {

    /** Ob Verzeichnisse akzeptiert werden sollen */
    private boolean _akzeptiereVerzeichnisse = true;

    /** Die Description */
    private String _beschreibung;

    /** Die Extension Liste */
    private String _extension;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erstellt einen neues <code>SimpleFileFilter</code> mit <code>_extension</code>
     * als Extension. _extension kann entweder nur eine Extension sein, z.B. "xml" oder
     * mehrere mit Kommas getrennt: "xml,xsl". Als Description wird automatisch
     * <code>_extension+"-Files (*.ext1, *.ext2)"</code> gesetzt, wobei ext1 und ext2
     * fr Element 1 und Element 2 der Extensionliste stehen. Es knnen natrlich
     * so viele Extensions wie gewnscht gesetzt werden.
     *
     * @param ext Die Liste mit den Extensions.
     */
    public SimpleFileFilter(String ext) {
        this._extension    = ext;
        this._beschreibung = ext + "-Files (" + generateExtensionString(ext) + ")";
    }

    /**
     * Erstellt einen neues <code>SimpleFileFilter</code> mit <code>_extension</code>
     * als Extension. _extension kann entweder nur eine Extension sein, z.B. "xml" oder
     * mehrere mit Kommas getrennt: "xml,xsl". Als Description wird automatisch
     * <code>_beschreibung+" (*.ext1, *.ext2)"</code> gesetzt, wobei ext1 und ext2
     * fr Element 1 und Element 2 der Extensionliste stehen. Es knnen natrlich
     * so viele Extensions wie gewnscht gesetzt werden.
     *
     * @param extension Die Liste mit den Extensions.
     * @param beschreibung Die Description
     */
    public SimpleFileFilter(String extension, String beschreibung) {
        this._extension    = extension;
        this._beschreibung = beschreibung + " (" + generateExtensionString(extension) + ")";
    }

    //~ METHODEN ==============================================================

    /**
     * Prft ob dieser <code>FileFilter</code> dieses <code>File</code> akzeptiert.
     * Dabei wird die Dateiendung mit jeder gesetzten Extension verglichen.
     *
     * @param f Das <code>File</code>
     * @return true wenn der FileFilter dieses File akzeptiert, false andererweise
     */
    public boolean accept(File f) {
        if (f.isDirectory() && _akzeptiereVerzeichnisse) {
            return true;
        }

        if (!isMultiExtensionString(_extension)) {
            return f.getName().endsWith(_extension);
        }

        String[] exts = _extension.split(",");

	    for (String ext : exts) {
		    if (f.getName().endsWith(ext)) {
			    return true;
		    }
	    }

        return false;
    }

    /**
     * Liefert einen String zurck, der als Description verwendet wird.
     * Der String wird aus einer Extensionliste generiert. Fr den String
     * "xml,xsl,txt" wrde der zurckgeliferte String dann so aussehen:
     * "*.xml, *.xsl, *.txt"
     *
     * @param ext Die Extensionliste
     * @return ein String der die Extensions darstellt.
     */
    protected String generateExtensionString(String ext) {
        String[]     exts = ext.split(",");
        StringBuffer sb   = new StringBuffer();

        for (int i = 0; i < exts.length; i++) {
            sb.append("*.");
            sb.append(exts[i]);

            if (i != exts.length - 1) {
                sb.append(",");
            }
        }

        return sb.toString();
    }

    //~ GET METHODEN ==========================================================

    /**
     * Liefert die Beschreibung zurck
     *
     * @return Die Beschreibung
     */
    public String getDescription() {
        return _beschreibung;
    }

    /**
     * Prft ob der bergebene Extensionstring mehrere Extensions beinhaltet.
     * Ein "Multi-Extensino-String" ist z.B. sowas: "xml,xsl,txt"
     *
     * @param ext Der String
     * @return true wenn der String mehrere Extensions beinhaltet
     */
    protected boolean isMultiExtensionString(String ext) {
        return ext.indexOf(',') >= 0;
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 22.04.10
