/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.praeferenzseiten;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.projektlib.speicher.GrundEinstellungen;
import de.inovat.buv.projektlib.speicher.OrdnerEinstellungen;

import org.eclipse.jface.preference.DirectoryFieldEditor;
import org.eclipse.jface.preference.FieldEditorPreferencePage;
import org.eclipse.ui.IWorkbench;
import org.eclipse.ui.IWorkbenchPreferencePage;

import static de.inovat.buv.projektlib.speicher.SpeicherParameter.BENUTZER_ORDNER;

//~ JDK IMPORTE ===============================================================

import java.util.HashSet;
import java.util.Set;

//~ KLASSEN ===================================================================

/**
 * Erzeugt PreferenzSeite fr die Einstellung des Ordners,
 * wo alle Projekt-Daten gespeichert werden.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 78 $ / $Date: 2013-10-30 16:28:58 +0100 (Mi, 30 Okt 2013) $ / ($Author: LG $)
 */
public class WorkbenchPreferencePageOrdner extends FieldEditorPreferencePage implements IWorkbenchPreferencePage {

    /** Menge der fr nderungen des Voreinstellungsordners registrierten Klassen */
    private static final Set<IOrdnerAenderung> mengeOrdnerAenderungsListener = new HashSet<IOrdnerAenderung>();

    //~ FELDER ================================================================

    private String               pfadAlt = "";
    private DirectoryFieldEditor pfad;

    //~ KONSTRUKTOREN  (und vom Konstruktor verwendete Methoden) ==============

    /**
     * Erzeugt eine Instanz der Klasse WorkbenchPreferencePageOrdner.
     */
    public WorkbenchPreferencePageOrdner() {
        setDescription("Whlen Sie einen Ordner, wo alle Projekt-Daten gespeichert werden:");
    }

    //~ METHODEN ==============================================================

    /**
     * Registriert Klasse als Listener auf {@link IOrdnerAenderung}.
     *
     * @param klasse
     *            Klasse, die das Interface {@link IOrdnerAenderung}
     *            implementiert und ber nderungen des Voreinstellungsordners
     *            informiert werden will.
     */
    public static void addOrdnerListener(IOrdnerAenderung klasse) {
        synchronized (mengeOrdnerAenderungsListener) {
            mengeOrdnerAenderungsListener.add(klasse);
        }
    }

    /**
     * Ruft bei nderungen des Voreinstellungsordners die Methode {#link
     * IOrdnerAenderung.voreinstellungsOrdnerGeaendert}() fr jede als Listener
     * registrierte Klasse auf.
     */
    private static void benachrichtigeListener() {
        synchronized (mengeOrdnerAenderungsListener) {
            for (IOrdnerAenderung klasse : mengeOrdnerAenderungsListener) {
                klasse.voreinstellungsOrdnerGeaendert();
            }
        }
    }

    @Override
    protected void createFieldEditors() {
        pfad = new DirectoryFieldEditor("", "Ordner", getFieldEditorParent());
        addField(pfad);
    }

    // --------------------- Interface IWorkbenchPreferencePage ---------------------
    @Override
    public void init(IWorkbench workbench) {}

    @Override
    protected void initialize() {
        super.initialize();

        String ordner = GrundEinstellungen.ermittleParameterWertFuerBenutzer(BENUTZER_ORDNER);

        if (ordner == null) {
            ordner = OrdnerEinstellungen.ermittleDefaultBenutzerOrdner();
        }

        pfad.setStringValue(ordner);
        pfadAlt = ordner;
    }

    protected void performDefaults() {
        super.performDefaults();
        pfad.setStringValue(OrdnerEinstellungen.ermittleDefaultBenutzerOrdner());
    }

    @Override
    public boolean performOk() {
        super.performOk();

        String ordner = pfad.getStringValue();
        
        GrundEinstellungen.speichereParameterFuerBenutzer(BENUTZER_ORDNER, ordner, "nderung des Projektausgabeordners fr den Benutzer");
        OrdnerEinstellungen.initialisiereBenutzerOrdner();

        // Falls die Daten gendert sind, benachrichtige Listener:
        if (!pfadAlt.equals(ordner)) {
            benachrichtigeListener();
        }

        return true;
    }

    /**
     * Lscht die Registrierung einer Klasse als Listener auf
     * {@link IOrdnerAenderung}.
     *
     * @param klasse
     *            Klasse, die sich zuvor mittels {#link addOrderListener}
     *            registriert hat.
     */
    public static void removeOrdnerListener(IOrdnerAenderung klasse) {
        synchronized (mengeOrdnerAenderungsListener) {
            mengeOrdnerAenderungsListener.remove(klasse);
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 12.03.13
