/*
 * Copyright (c) 2008 by inovat, Dipl.-Ing. H. C. Kni
 * ALL RIGHTS RESERVED.
 *
 * THIS SOFTWARE IS  PROVIDED  "AS IS"  AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF  MERCHANTABILITY  AND  FITNESS  FOR  A PARTICULAR  PURPOSE ARE
 * DISCLAIMED. IN NO EVENT SHALL inovat OR ITS CONTRIBUTORS BE
 * LIABLE FOR ANY  DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
 * OR CONSEQUENTIAL DAMAGES  (INCL., BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES;  LOSS OF USE, DATA, OR  PROFITS;
 * OR BUSINESS INTERRUPTION)  HOWEVER  CAUSED  AND  ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,  OR TORT (INCL.
 * NEGLIGENCE OR OTHERWISE)  ARISING  IN  ANY  WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */



package de.inovat.buv.projektlib.speicher;

//~ NICHT JDK IMPORTE =========================================================

import de.inovat.buv.inovat.lib.debug.Log;
import de.inovat.buv.projektlib.Activator;
import de.inovat.buv.projektlib.bundesland.BundeslandVew;
import de.inovat.buv.projektlib.dav.DavDatenVew;
import de.inovat.buv.projektlib.konstanten.AufrufParameter;
import de.inovat.buv.projektlib.konstanten.Zustaende;

import org.eclipse.jface.dialogs.MessageDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.PlatformUI;

import static de.inovat.buv.inovat.lib.konstanten.Konstanten.FILESEPARATOR;
import static de.inovat.buv.inovat.lib.konstanten.Konstanten.NEWLINE;

//~ JDK IMPORTE ===============================================================

import java.io.File;
import java.io.IOException;

//~ KLASSEN ===================================================================

/**
 * Klasse ermittelt den Ausgabe Ordner fr das Projekt.
 *
 * @author inovat
 * @author Liliya Givorgizova (LG)
 * @version $Revision: 38 $ / $Date: 2013-10-08 15:30:34 +0200 (Di, 08 Okt 2013) $ / ($Author: LG $)
 */
public class OrdnerEinstellungen {

    /** Temp-Ordner. */
    private final static String ORDNER_TEMP = "inovatTemp";

    /**
     * Liegt fest, ob der AusgabeOrdner mit den Unterordnern
     * (Projektname_Asugabe\Bundesland) erstellt werden soll.
     */
    public static final boolean IST_AUSGABE_UNTER_ORDNER_EINGESCHALTET;

    /**
     * Name des in der Prferenz eingegebenen Benutzer-Ordners,
     * wo Projekt-AusgabeDateien gespeichert werden.
     */
    private static String ORDNER_BENUTZER;

    /** UnterOrdner fr ProjektAusgabe. */
    private final static String UNTERORDNER_PROJEKT_AUSGABE;

    //~ STATISCHE INITIALISIERER ==============================================

    static {

        // --------------------------------------------------------------------
        // Ermittle, ob der AusgabeOrdner mit den Unterordnern erstellt werden soll:
        boolean wert = DavDatenVew.getInstanz().getAufrufParameterAlsBoolean(AufrufParameter.AUSGABE_UNTER_ORDNER, false);

        IST_AUSGABE_UNTER_ORDNER_EINGESCHALTET = wert;

        // --------------------------------------------------------------------
        // Definiere den ProjektAusgabe-Unterordner:
        if (IST_AUSGABE_UNTER_ORDNER_EINGESCHALTET) {
            UNTERORDNER_PROJEKT_AUSGABE = String.format("%s_Ausgabe%s%s", Zustaende.PROJEKT_NAME, FILESEPARATOR, BundeslandVew.getInstanz().getBundesland().getAbkuerzung());
        } else {
            UNTERORDNER_PROJEKT_AUSGABE = "";
        }

        // --------------------------------------------------------------------
        // Initialisiere den Benutzer-Ordner:
        initialisiereBenutzerOrdner();
    }

    //~ METHODEN ==============================================================

    /**
     * Ermittle den DefaultBenutzerOrdner fr die Projekt-AusgabeDateien.
     *
     * @return DefaultWert fr den Benutzerordner mit den Projekt-AusgabeDateien.
     */
    public static String ermittleDefaultBenutzerOrdner() {

        // ----------------------------------------------------------------
        // Hole das KommandoZeilenParameter fr den DefaultBenutzerOrdner:
        String ordner = DavDatenVew.getInstanz().getAufrufParameterAlsString(AufrufParameter.DEFAULT_BENUTZER_ORDNER);

        // ----------------------------------------------------------------
        // Falls kein Parameter bergeben wird:
        if (ordner == null) {

            // user.home:
            ordner = System.getProperty("user.home");
        }

        // ----------------------------------------------------------------
        // Falls der Ordner nicht initialisiert wird:
        if (ordner == null) {
            ordner = String.format("D:%s", FILESEPARATOR);
        }

        return ordner;
    }

    /**
     * Ermittelt den Ordner, wo alle Projekt-AusgabeDateien gespeichert sind
     * (ohne FILESEPARATOR am Ende).
     * Der Ordner wird immer neu ermittelt, um die nderung von ORDNER_BENUTZER
     * ber die Preferenzseite zu bercksichtigen.
     *
     * @return Ordner, wo alle Projekt-AusgabeDateien gespeichert sind.
     */
    private static String ermittleProjektAusgabenOrdner() {
        String ordner;

        // Falls der Benutzerordner FILESEPARATOR am Ende hat:
        if (ORDNER_BENUTZER.lastIndexOf(FILESEPARATOR) == ORDNER_BENUTZER.length() - FILESEPARATOR.length()) {
            if (IST_AUSGABE_UNTER_ORDNER_EINGESCHALTET) {
                ordner = String.format("%s%s", ORDNER_BENUTZER, UNTERORDNER_PROJEKT_AUSGABE);
            } else {
                ordner = ORDNER_BENUTZER.substring(0, ORDNER_BENUTZER.length() - FILESEPARATOR.length());
            }
        }

        // Falls der Benutzerordner kein FILESEPARATOR am Ende hat:
        else {
            if (IST_AUSGABE_UNTER_ORDNER_EINGESCHALTET) {
                ordner = String.format("%s%s%s", ORDNER_BENUTZER, FILESEPARATOR, UNTERORDNER_PROJEKT_AUSGABE);
            } else {
                ordner = ORDNER_BENUTZER;
            }
        }

        //
        erzeugeOrdner(ordner);

        return ordner;
    }

    /**
     * Ermittelt den Ordner mit angegebenen Namen, der im Projekt-
     * AusgabeOrdner liegt.
     *
     * @param unterOrdner Name des Ordner mit Fileseparator vorne.
     * Soll in diese Klasse als Konstante definiert werden.
     *
     * @return Ordner mit angegebenen Namen, der im Projekt-
     * AusgabeOrdner liegt.
     */
    public static String ermittleProjektAusgabenUnterOrdner(String unterOrdner) {
        String ordner = String.format("%s%s%s", ermittleProjektAusgabenOrdner(), FILESEPARATOR, unterOrdner);

        erzeugeOrdner(ordner);

        return ordner;
    }

    /**
     * Ermittelt den Ordner, wo Temp-Dateien gespeichert werden.
     * @return Ordner, wo Temp-Dateien gespeichert werden.
     */
    public static String ermittleTempOrdner() {
        String ordner;

        if (ORDNER_BENUTZER.lastIndexOf(FILESEPARATOR) == ORDNER_BENUTZER.length() - FILESEPARATOR.length()) {
            ordner = String.format("%s%s", ORDNER_BENUTZER, ORDNER_TEMP);
        } else {
            ordner = String.format("%s%s%s", ORDNER_BENUTZER, FILESEPARATOR, ORDNER_TEMP);
        }

        erzeugeOrdner(ordner);

        return ordner;
    }

    /**
     * Erzeugt den Ordner, falls der noch nicht existiert.
     *
     * @param ordner Pfad und Name vom Ordner, der erzeugt werden soll.
     *
     * @return Ergebnis, ob der Ordner erzeugt wurde.
     */
    public static boolean erzeugeOrdner(String ordner) {
        File file = new File(ordner);

        if (!file.exists()) {
            try {

                // noinspection ResultOfMethodCallIgnored
                file.mkdirs();

                return true;
            }
            catch (SecurityException e) {
                Log.zeigeInterneMultiMeldung(Activator.PLUGIN_ID, String.format("Der Ordner <%s> kann nicht erstellt werden!", ordner), e);

                return false;
            }
        }

        return true;
    }

    /**
     * Initialisiere den BenutzerOrdner,
     * wo der Ordner mit der Projekt-AusgabeDateien erzeugt wird.
     * Der Ordner wird auf Schreibrechte geprft.
     */
    public static void initialisiereBenutzerOrdner() {
        String ordner = GrundEinstellungen.ermittleParameterWertFuerBenutzer(SpeicherParameter.BENUTZER_ORDNER);

        if (ordner == null) {
            ordner = ermittleDefaultBenutzerOrdner();
        }

        //
        ORDNER_BENUTZER = ordner;

        //
        erzeugeOrdner(ordner);

        // Prfe, ob der Benutzter fr den angegebenen Ordner die Schreibrechte hat:
        if (!istOrdnerSchreibbar(ermittleTempOrdner())) {
            final String text = String.format("Sie haben keine Schreibrechte fr den Ordner <%s>!%s%sBitte, einen anderen Ordner fr die Einstellungen whlen!%s%sNeuer Ordner kann ber die Prferenzen -> inovat -> Datei Speicherstelle definiert werden.", ordner, NEWLINE, NEWLINE, NEWLINE, NEWLINE);

            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, text));

            try {
                MessageDialog.openError(PlatformUI.getWorkbench().getActiveWorkbenchWindow().getShell(), "Error", text);
            }
            catch (Exception e) {

                // Falls in diesem Thread noch kein aktives WorkbenchWindow vorhanden ist,
                // hole das von einem anderen Thread:
                PlatformUI.getWorkbench().getDisplay().asyncExec(new Runnable() {
                    @Override
                    public void run() {
                        try {
                            Shell shell = PlatformUI.getWorkbench().getWorkbenchWindows()[0].getShell();

                            MessageDialog.openError(shell, "Error", text);
                        }
                        catch (Exception e2) {
                            Log.zeigeInterneMeldung(Activator.PLUGIN_ID, e2);
                        }
                    }
                });
            }
        }
    }

    /**
     * berprfe, ob der User die Schreibrechte fr den Ordner hat.
     *
     * @param ordnerName Ordner, fr den die Schreibrechte berprfen werden sollen.
     *
     * @return true, falls der User die Schreibrechte fr den Ordner hat, false - sonst.
     */
    private static boolean istOrdnerSchreibbar(String ordnerName) {

        // Erzeuge die temporre Datei und falls geschafft, gebe true zurck.
        boolean ergebnis = true;
        File    ordner   = new File(ordnerName);

        try {
            File.createTempFile("datei_", null, ordner).deleteOnExit();
        }
        catch (IOException e) {
            ergebnis = false;
        }

        return ergebnis;
    }

    /**
     * Lsche alle Dateien im Tempordner.
     */
    public static void loescheTempOrdner() {
        try {
            File   ordner = new File(ermittleTempOrdner());
            File[] ar     = ordner.listFiles();

            if (ar != null) {
                for (File datei : ar) {
                    datei.delete();
                }
            }

            //
            ordner.delete();
        }
        catch (Error e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Dateien im Tempordner knnen nicht gelscht werden!", e));
        }
        catch (Exception e) {
            Log.zeige(Log.erzeugeMeldung(Log.FEHLER, Activator.PLUGIN_ID, "Die Dateien im Tempordner knnen nicht gelscht werden!", e));
        }
    }
}


//~Formatiert mit 'inovat Kodierkonvention' am 08.10.13
